-----------------------------------------------------------------------------------------------
-- Client Lua Script for AnnouncementManager
-- Copyright (c) NCsoft. All rights reserved
--
-- Ecrit par Elvoniel pour la guilde Exile Diamant serveur Treespeaker 
-- AnnouncementManager version 2.3
-- 
-- Ecrit le 07/08/2014
--
-- N'hsitez pas  me contacter pour des reports de bugs, questions, suggestions, demandes d'aide
--  l'adresse suivante : elvoniel@outlook.fr
-----------------------------------------------------------------------------------------------
 
require "Window"
require "ChatSystemLib"
 
-----------------------------------------------------------------------------------------------
-- AnnouncementManager Dfinition du module
-----------------------------------------------------------------------------------------------
local AnnouncementManager = {} 
 
-----------------------------------------------------------------------------------------------
-- Constantes
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999

local selectedTextColor = ApolloColor.new("ConModerate") -- Correspond  la couleur dore
local normalTextColor = ApolloColor.new("AddonLoaded") -- Correspond  la couleur blanche (par dfaut)
local grayTextColor = ApolloColor.new("darkgray")
 
-----------------------------------------------------------------------------------------------
-- Fonctions d'initialisation
-----------------------------------------------------------------------------------------------

function AnnouncementManager:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here
	o.itemList = {} -- Ici on initialise le contenu de la liste des items affichs
	o.announcementList = {} -- On initialise le contenu des annonces en mmoire
	o.autoConfiguration = nil
	o.selectedItem = nil -- On retient quel item (affich) a t slectionn
	o.editingItem = nil -- On retient le titre de l'annonce en cours d'dition
	o.currentMap = nil


    return o
end

function AnnouncementManager:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
	Apollo.RegisterEventHandler("GenericEvent_QuestLink", "OnLink", self)
	Apollo.RegisterEventHandler("ItemLink", "OnLink", self)
	Apollo.RegisterEventHandler("GenericEvent_ArchiveArticleLink", "OnLink", self)
	
end
 


-- OnLoad : Dclenche au chargement de l'addon
function AnnouncementManager:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("AnnouncementManager.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end


-- OnDocLoaded : Dclenche quand le doctument XML a t charg
function AnnouncementManager:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "AnnouncementManagerForm", nil, self)
		self.editForm = Apollo.LoadForm(self.xmlDoc, "EditForm", nil, self)
		self.autoForm = Apollo.LoadForm(self.xmlDoc, "AutoForm", nil, self)
		
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		
		self.wndItemList = self.wndMain:FindChild("ItemList") -- On rcupre la liste des items (le widget)
					
	    self.wndMain:Show(false, true)
		self.editForm:Show(false, true)
		self.autoForm:Show(false, true)

		-- if the xmlDoc is no longer needed, you should set it to nil
		-- self.xmlDoc = nil
		
		-- Register handlers for events, slash commands and timer, etc.
		-- e.g. Apollo.RegisterEventHandler("KeyDown", "OnKeyDown", self)
		Apollo.RegisterSlashCommand("an", "OnAnnouncementManagerOn", self)
		
		self.generalTimer = ApolloTimer.Create(60.000, true, "OnTimer", self) -- On cre un timer rptable de 1 Minute
		self.timerZone = ApolloTimer.Create(1.000, true, "CheckZoneChanged", self) -- On cre un timer rptable de 1 seconde
	
		-- Do additional Addon initialization here
		self:InitCB()
		self.currentMap  = ""
	end
end


-----------------------------------------------------------------------------------------------
-- AnnouncementManager Save/Load (fonctions appeles lorsque l'on quitte le jeu et qu'on le lance)
-----------------------------------------------------------------------------------------------

function AnnouncementManager:OnSave(eLevel)
  if eLevel ~= GameLib.CodeEnumAddonSaveLevel.Character then
    return nil
  end

  local tSavedData = { } -- Cette structure contiendra les donnes  sauvegarder

  for title, content in pairs(self.announcementList) do
	tSavedData[title] = content -- On remplit la structure  sauvegarder (copie de la liste des annonces)
  end

  return tSavedData
end

function AnnouncementManager:OnRestore(eLevel, tSavedData)
  if tSavedData ~= nil then

    for title, content in pairs(tSavedData) do
		self.announcementList[title] = content -- Si des donnes sauvegardes existent alors on restaure la liste des annonces
	end
  end
end


-----------------------------------------------------------------------------------------------
-- Fonctions de l'AnnouncementManagerForm 
-----------------------------------------------------------------------------------------------


-- when the OK button is clicked
function AnnouncementManager:OnOK()
	self.wndMain:Close() -- hide the window
end

-- when the Cancel button is clicked
function AnnouncementManager:OnCancel()
	self.wndMain:Close() -- hide the window
end

-- OnAnnounce : fonction appelle lors de l'appui sur le bouton "Annoncer"
function AnnouncementManager:OnAnnounce()

	if self.selectedItem ~= nil then
	
		local title = self.selectedItem:FindChild("Text"):GetText()
		
		self:Announce(title)
		
	else
		self:Info("You have to select an announcement before doing that!")		
	end
end


-- OnAdd : fonction appele lorsque le boutton "Ajouter" de la fentre principale est press
function AnnouncementManager:OnAdd()
	self:OpenEditForm("Add") -- On ouvre la fentre d'dition / ajout en mode ajout
end

-- PopulateItemList : permet de remplir la liste avec les annonces dans la fentre principale
function AnnouncementManager:PopulateItemList()

	self:DestroyItemList() -- On dtruit la liste prcdente

	for title, content in pairs(self.announcementList) do
		self:AddAnnounceOnDisplay(title) -- On ajoute les annonces une par une dans la liste
	end
	
	self.wndItemList:ArrangeChildrenVert() -- On arrange la liste de manire verticale
	
end


-- DestroyItemList : permet de supprimer la liste (des items affichs) prcdente
function AnnouncementManager:DestroyItemList()

	for title, wnd in pairs(self.itemList) do
		wnd:Destroy() -- On dtruit le widget ListItem pour chaque ListItem contenu dans la liste ItemList
	end
	
	self.itemList = {} -- On rinitialise la liste des items
	self.selectedItem = nil 
end

-- AddAnnounceOnDisplay : permet d'ajouter une annonce dans la liste des items
-- title : titre de l'annonce  ajouter dans la liste
function AnnouncementManager:AddAnnounceOnDisplay(title)

	local wnd = Apollo.LoadForm(self.xmlDoc, "ListItem", self.wndItemList, self) -- On cre une nouvelle ListItem que l'on positionne dans l'emplacement prvu pour contenir les annonces (widget ItemList)
	
	self.itemList[title] = wnd -- On enregistre cet ListItem
	
	local wndItemText = wnd:FindChild("Text")
	wndItemText:SetText(title) -- on modifie le text affich
	
	if self.announcementList[title][3] then -- Si l'annonce est automatique alors on place une petite icone devant
		wnd:FindChild("Picture"):Show(true)
	end

end


-- OnListItemSelected : appele quand l'utilisateur clique sur un ListItem
function AnnouncementManager:OnListItemSelected( wndHandler, wndControl, eMouseButton, nLastRelativeMouseX, nLastRelativeMouseY )
	if wndHandler ~= wndControl then
	   return
	end
	
	local wndItemText
	
	-- Ce qui suit permet de remettre la couleur normale  l'lment prcdement slectionn
	if self.selectedItem ~= nil then
		wndItemText = self.selectedItem:FindChild("Text")
		wndItemText:SetTextColor(normalTextColor)
	end
	
	self.selectedItem = wndControl -- Permet de sauvegarder le ListItem slectionn
	
	-- Ce qui suit permet de colorer le nouvel lment slectionn
	wndItemText = wndControl:FindChild("Text")
		
	wndItemText:SetTextColor(selectedTextColor)

end

---------------------------------------------------------------------------------------------------
-- Fonctions de l'EditForm
---------------------------------------------------------------------------------------------------


-- OpenEditForm : permet de charger la fentre d'ajout / dition
-- action : Le mode d'ouverture de la fentre (Ajouter ou Editer)
function AnnouncementManager:OpenEditForm(action)

	self.editForm:Invoke() -- On appelle la fentre
	
	local title = self.editForm:FindChild("Title")
	
	local btnAdd = self.editForm:FindChild("BtnAdd")
	local btnEdit = self.editForm:FindChild("BtnEdit")
	
	-- On masque les deux bouttons (Ajouter et supprimer) pour afficher le bon par la suite en fonction du mode
	btnAdd:Show(false) 
	btnEdit:Show(false)
	
	if action == "Add" then
		btnAdd:Show(true)
	else
		btnEdit:Show(true)
	end
	
	title:SetText(action) -- On change le titre de la fentre en fonction du mode
	
	self.autoConfiguration = nil
	
	self.editForm:FindChild("BtnAuto"):Enable(false)

end

-- OnEditClose : appelle quand la fentre d'Ajout / Edition est ferme
function AnnouncementManager:OnEditClose( wndHandler, wndControl, eMouseButton )

	-- On rinitialise le contenu des TextBox
	self.editForm:FindChild("EditTextTitle"):FindChild("EditBox"):SetText("") 
	self.editForm:FindChild("EditTextContent"):FindChild("EditBox"):SetText("")
	
	self:UnCheckAll() -- On dcoche les CheckBox d'options de porte
		
	self.editForm:Close() -- On ferme la fentre
	
	if self.autoForm:IsShown() then -- Si la fentre de configuration des annonces automatiques est ouverte alors on la ferme aussi
		self:OnCloseAutoForm()
	end
	
	self.autoConfiguration = nil
	self.editingItem = nil
end

-- OnEnableConfigBtn : Appele lors de le coche ou de la dcoche de la CHeckBox "Set automatic"
function AnnouncementManager:OnEnableConfigBtn()

	local btn = self.editForm:FindChild("BtnAuto")
	
	if btn:IsEnabled() then
		btn:Enable(false)
	else
		btn:Enable(true)
	end
	
end

function AnnouncementManager:EnableCircles()
	self.editForm:FindChild("EditBoxCircles"):Show(true)
	self.editForm:FindChild("CirclesDisabled"):Show(false)
end

function AnnouncementManager:DisableCircles()
	self.editForm:FindChild("CirclesDisabled"):Show(true)
	self.editForm:FindChild("EditBoxCircles"):Show(false)
end

---------------------------------------------------------------------------------------------------
-- Fonctions de l'AutoForm
---------------------------------------------------------------------------------------------------

-- OnAutoConfigure : Appele lors de l'appui sur le bouton "Set auto config". Permet d'initialiser la fentre de configuration
function AnnouncementManager:OnAutoConfigure()

	local configGroup = self.autoForm:FindChild("ConfigGroup")
	local title = self.editingItem
	
	self:DisableZoneGroup()
	
	if self.autoConfiguration ~= nil then -- Si une configuration est en cour mais n'a pas encore t valide
	
		configGroup:FindChild("EditBoxRepeat"):FindChild("EditBox"):SetText(self.autoConfiguration.RepeatTime)
		configGroup:FindChild("HourZone"):SetText(string.sub(self.autoConfiguration.ScheduleOn, 1, 2))
		configGroup:FindChild("MinuteZone"):SetText(string.sub(self.autoConfiguration.ScheduleOn, 4))
		configGroup:FindChild("CBEnableRepeat"):SetCheck(self.autoConfiguration.EnableRepeat)
		configGroup:FindChild("CBEnableSchedule"):SetCheck(self.autoConfiguration.EnableSchedule)
		
		self.autoForm:FindChild("ZoneGroup"):FindChild("ZoneActivated"):FindChild("EditBoxZone"):SetText(self.autoConfiguration.Zone)
		self.autoForm:FindChild("LimitZone"):FindChild("EditBox"):SetText(self.autoConfiguration.Limit)
		
		if self.autoConfiguration.Zone ~= "" then
			self.autoForm:FindChild("ZoneActivated"):SetText("")
		end

		if self.autoConfiguration.EnableZoneChanged then
			configGroup:FindChild("BtnZoneChanged"):SetCheck(true)
			self:EnableZoneGroup()	
		end

	else
	
		if self.editingItem  ~= nil then -- Si un lment a t slectionn
			
			if self.announcementList[title][3] then -- On affiche les infos relatives  cet lment
				configGroup:FindChild("EditBoxRepeat"):FindChild("EditBox"):SetText(self.announcementList[title][4].RepeatTime)
				configGroup:FindChild("HourZone"):SetText(string.sub(self.announcementList[title][4].ScheduleOn, 1, 2))
				configGroup:FindChild("MinuteZone"):SetText(string.sub(self.announcementList[title][4].ScheduleOn, 4))
				configGroup:FindChild("CBEnableRepeat"):SetCheck(self.announcementList[title][4].EnableRepeat)
				configGroup:FindChild("CBEnableSchedule"):SetCheck(self.announcementList[title][4].EnableSchedule)
				
				self.autoForm:FindChild("ZoneGroup"):FindChild("ZoneActivated"):FindChild("EditBoxZone"):SetText(self.announcementList[title][4].Zone)
				self.autoForm:FindChild("LimitZone"):FindChild("EditBox"):SetText(self.announcementList[title][4].Limit)
				
				if self.announcementList[title][4].Zone ~= "" then
					self.autoForm:FindChild("ZoneActivated"):SetText("")
				end

				if self.announcementList[title][4].EnableZoneChanged then
					configGroup:FindChild("BtnZoneChanged"):SetCheck(true)
					self:EnableZoneGroup()				
				end
						
			end			
		end
	end
	
	-- On affiche les champs correspondants aux options "repeat" et "Schedule" si besoin
	if not configGroup:FindChild("CBEnableRepeat"):IsChecked() then
		self:DisableRepeat()
	else
		self:EnableRepeat()
	end
	
	if not configGroup:FindChild("CBEnableSchedule"):IsChecked() then
		self:DisableSchedule()
	else
		self:EnableSchedule()
	end

	self.autoForm:Invoke()

end

-- OnCloseAutoForm : Appele lors de la fermeture de l'AutoForm
function AnnouncementManager:OnCloseAutoForm()

	self.autoForm:Close()
	
	-- On rinitialise l'tat de la fentre
	self.autoForm:FindChild("ConfigGroup"):FindChild("CBEnableRepeat"):SetCheck(false)
	self.autoForm:FindChild("ConfigGroup"):FindChild("EditBoxRepeat"):FindChild("EditBox"):SetText("")
	self.autoForm:FindChild("ConfigGroup"):FindChild("CBEnableSchedule"):SetCheck(false)
	self.autoForm:FindChild("ConfigGroup"):FindChild("HourZone"):SetText("12")
	self.autoForm:FindChild("ConfigGroup"):FindChild("MinuteZone"):SetText("00")
	self.autoForm:FindChild("ConfigGroup"):FindChild("BtnZoneChanged"):SetCheck(false)
	self.autoForm:FindChild("ZoneActivated"):SetText("Any Zone")
	self.autoForm:FindChild("EditBoxZone"):SetText("")
	self.autoForm:FindChild("LimitZone"):FindChild("EditBox"):SetText("0")
	
end

-- OnHoursUp : Fonction appele lors de l'appui sur le bouton "Flche haute" pour les heures
function AnnouncementManager:OnHoursUp()

	local hourZone = self.autoForm:FindChild("ScheduleGroup"):FindChild("HourZone")
	local hour = hourZone:GetText()
	local newHour
	
	if hour == "23" then
		newHour = "0"
	else
		newHour = hour + 1
	end
	
	newHour = self:CheckDigits(newHour) -- On met l'heure sur deux digits
	hourZone:SetText(newHour)
	
end

-- OnHoursDown: Fonction appele lors de l'appui sur le bouton "Flche basse" pour les heures
function AnnouncementManager:OnHoursDown()

	local hourZone = self.autoForm:FindChild("ScheduleGroup"):FindChild("HourZone")
	local hour = hourZone:GetText()
	local newHour
	
	if hour == "00" then
		newHour = "23"
	else
		newHour = hour - 1
	end
	
	newHour = self:CheckDigits(newHour)
	hourZone:SetText(newHour)
	
end

-- OnMinutesUp: Fonction appele lors de l'appui sur le bouton "Flche haute" pour les minutes
function AnnouncementManager:OnMinutesUp()

	local minuteZone = self.autoForm:FindChild("ScheduleGroup"):FindChild("MinuteZone")
	local minutes = minuteZone:GetText()
	local newMinutes
	
	if minutes == "59" then
		newMinutes = "0"
	else
		newMinutes = minutes + 1
	end
	
	newMinutes = self:CheckDigits(newMinutes)
	minuteZone:SetText(newMinutes)
	
end

-- OnMinutesUp: Fonction appele lors de l'appui sur le bouton "Flche basse" pour les minutes
function AnnouncementManager:OnMinutesDown()

	local minuteZone = self.autoForm:FindChild("ScheduleGroup"):FindChild("MinuteZone")
	local minutes = minuteZone:GetText()
	local newMinutes
	
	if minutes == "00" then
		newMinutes = "59"
	else
		newMinutes = minutes - 1
	end
	
	newMinutes = self:CheckDigits(newMinutes)
	minuteZone:SetText(newMinutes)
	
end

-- OnSaveConfig : Appele lors de l'appui sur le bouton "Save config" de l'AutoForm. Permet de sauvegarder la configuration d'une annonce automatique
function AnnouncementManager:OnSaveConfig()

	local repeatField = self.autoForm:FindChild("ConfigGroup"):FindChild("EditBoxRepeat"):FindChild("EditBox")
	local zoneField = self.autoForm:FindChild("ZoneGroup"):FindChild("ZoneActivated"):FindChild("EditBoxZone")
	local limitTimeField = self.autoForm:FindChild("LimitZone"):FindChild("EditBox")
	
	local repeatTime = repeatField:GetText()
	local limitTime = limitTimeField:GetText()
	local scheduledOn = self.autoForm:FindChild("HourZone"):GetText() .. ":" .. self.autoForm:FindChild("MinuteZone"):GetText() -- on cre une chane de caractre pour l'heure de la forme "10:20"
	local whenZoneChanged = false
	local enableRepeat = false
	local enableSchedule = false
	local zone = zoneField:GetText()
	
	if self.autoForm:FindChild("BtnZoneChanged"):IsChecked() then
		whenZoneChanged = true
	end
	
	if self.autoForm:FindChild("CBEnableRepeat"):IsChecked() then
		enableRepeat = true
	end

	if self.autoForm:FindChild("CBEnableSchedule"):IsChecked() then
		enableSchedule = true
	end
	
	-- Si l'option "repeat" a t coche et que le contenu du champs n'est pas un nombre ou est infrieur ou gale  0, on affiche un message d'erreur
	if enableRepeat and (not tonumber(repeatTime) or tonumber(repeatTime) <= 0) then
		repeatField:SetText("")
		self:Info("You have to enter a number of minutes superior to 0 in the repeat field!")
	elseif whenZoneChanged and (not tonumber(limitTime) or tonumber(limitTime) < 0) then
		limitTimeField:SetText("")
		self:Info("You have to enter a number of minutes superior or equal to 0 in the limit field!")
	else
		self.autoConfiguration = {EnableRepeat = enableRepeat, EnableSchedule = enableSchedule, EnableZoneChanged = whenZoneChanged, Zone = zone, RepeatCounter = 0, LimitCounter = limitTime, Limit = limitTime, RepeatTime = repeatTime, ScheduleOn = scheduledOn}
		self:OnCloseAutoForm()
	end
end

function AnnouncementManager:SetCurrentZone()

	local currentZoneName= GameLib.GetCurrentZoneMap().strName	
	
	self.autoForm:FindChild("ConfigGroup"):FindChild("ZoneGroup"):FindChild("EditBoxZone"):SetText(currentZoneName)
	self:HideZoneText()
end

function AnnouncementManager:EnableRepeat()
	self.autoForm:FindChild("ConfigGroup"):FindChild("EditBoxRepeat"):Show(true)
	self.autoForm:FindChild("ConfigGroup"):FindChild("RepeatDisabled"):Show(false)
end

function AnnouncementManager:DisableRepeat()
	self.autoForm:FindChild("ConfigGroup"):FindChild("EditBoxRepeat"):Show(false)
	self.autoForm:FindChild("ConfigGroup"):FindChild("RepeatDisabled"):Show(true)
end

function AnnouncementManager:EnableSchedule()
	-- self.autoForm:FindChild("ConfigGroup"):FindChild("ScheduleGroup"):Show(true)	
	self.autoForm:FindChild("BtnHup"):Enable(true)
	self.autoForm:FindChild("BtnHdown"):Enable(true)
	self.autoForm:FindChild("BtnMup"):Enable(true)
	self.autoForm:FindChild("BtnMdown"):Enable(true)
	
	self.autoForm:FindChild("HourZoneDisabled"):Show(false)
	self.autoForm:FindChild("MinuteZoneDisabled"):Show(false)
	
	self.autoForm:FindChild("HourZone"):Show(true)
	self.autoForm:FindChild("MinuteZone"):Show(true)
end

function AnnouncementManager:DisableSchedule()
	-- self.autoForm:FindChild("ConfigGroup"):FindChild("ScheduleGroup"):Show(false)
	self.autoForm:FindChild("BtnHup"):Enable(false)
	self.autoForm:FindChild("BtnHdown"):Enable(false)
	self.autoForm:FindChild("BtnMup"):Enable(false)
	self.autoForm:FindChild("BtnMdown"):Enable(false)
	
	self.autoForm:FindChild("HourZoneDisabled"):Show(true)
	self.autoForm:FindChild("MinuteZoneDisabled"):Show(true)
	
	self.autoForm:FindChild("HourZone"):Show(false)
	self.autoForm:FindChild("MinuteZone"):Show(false)
end

function AnnouncementManager:EnableZoneGroup()
	self.autoForm:FindChild("ConfigGroup"):FindChild("ZoneGroup"):FindChild("BtnSelectZone"):Enable(true)
	self.autoForm:FindChild("ConfigGroup"):FindChild("ZoneGroup"):FindChild("ZoneActivated"):Show(true)
	self.autoForm:FindChild("ConfigGroup"):FindChild("ZoneGroup"):FindChild("ZoneDisactivated"):Show(false)
	self.autoForm:FindChild("ConfigGroup"):FindChild("LimitZoneDisabled"):Show(false)
	self.autoForm:FindChild("ConfigGroup"):FindChild("LimitZone"):Show(true)
	
	self.autoForm:FindChild("ConfigGroup"):FindChild("LabelLimit"):SetTextColor(normalTextColor)
	self.autoForm:FindChild("ConfigGroup"):FindChild("LabelMinutes"):SetTextColor(normalTextColor)

end

function AnnouncementManager:DisableZoneGroup()
	self.autoForm:FindChild("ConfigGroup"):FindChild("ZoneGroup"):FindChild("BtnSelectZone"):Enable(false)
	self.autoForm:FindChild("ConfigGroup"):FindChild("ZoneGroup"):FindChild("ZoneActivated"):Show(false)
	self.autoForm:FindChild("ConfigGroup"):FindChild("ZoneGroup"):FindChild("ZoneDisactivated"):Show(true)
	self.autoForm:FindChild("ConfigGroup"):FindChild("LimitZoneDisabled"):Show(true)
	self.autoForm:FindChild("ConfigGroup"):FindChild("LimitZone"):Show(false)
	
	self.autoForm:FindChild("ConfigGroup"):FindChild("LabelLimit"):SetTextColor(grayTextColor) 
	self.autoForm:FindChild("ConfigGroup"):FindChild("LabelMinutes"):SetTextColor(grayTextColor)
end

function AnnouncementManager:HideZoneText()
	self.autoForm:FindChild("ConfigGroup"):FindChild("ZoneGroup"):FindChild("ZoneActivated"):SetText("")
end

-----------------------------------------------------------------------------------------------
-- Fonctions gnrales
-----------------------------------------------------------------------------------------------
-- Define general functions here

-- on SlashCommand "/an"
function AnnouncementManager:OnAnnouncementManagerOn()
	self.wndMain:Invoke() -- show the window
	self:PopulateItemList() -- Initialise le contenu de la liste des items (affichs)
end

-- OnTimer : Appele toute les 1 minute
function AnnouncementManager:OnTimer()

	local localTime = GameLib:GetLocalTime() -- On rcupre l'heure locale
	local localTimeString = ""
	
	localTimeString = string.format("%02d:%02d", tostring(localTime.nHour), tostring(localTime.nMinute)) or "" -- On convertie l'heure locale en string
	
	for title, content in pairs(self.announcementList) do
		if content[3] then -- Si l'annonce est automatique
			if content[4].EnableRepeat then -- Si elle est rptable
				self:HandleRepeat(title, content[4])
			end
			if content[4].EnableSchedule then -- Si elle est programme
				self:HandleSchedule(title, content[4], localTimeString)
			end	
			if content[4].EnableZoneChanged and tonumber(content[4].Limit) ~= 0 and tonumber(content[4].LimitCounter) < tonumber(content[4].Limit) then
				content[4].LimitCounter = content[4].LimitCounter + 1
			end	
		end
	end
	
end


-- OnDelete : fonction appele lors de l'appui sur le bouton "Supprimer"
--			  Permet la suppression d'une annonce
function AnnouncementManager:OnDelete()

	if self.selectedItem ~= nil then	
		
		local title = self.selectedItem:FindChild("Text"):GetText() -- On rcupre le titre de l'annonce qui est le text de l'item slectionn
		
		self.announcementList[title] = nil -- Le titre tant la clef, on a juste  supprimer l'annonce dans la structure
		
		self:PopulateItemList() -- On rafraichit l'affichage de la liste
		
		self.selectedItem = nil
	else
		self:Info("You have to select an announcement before doing that!")
	end

end

-- addAnnouncement : fonction permettant l'ajout d'une nouvelle annonce
function AnnouncementManager:addAnnouncement()

	local titleText = self.editForm:FindChild("EditTextTitle"):FindChild("EditBox"):GetText() -- On rcupre le texte contenu dans la premire TextBox (le titre)
	local contentText = self.editForm:FindChild("EditTextContent"):FindChild("EditBox"):GetText() -- On rcupre le texte contenu dans la seconde TextBox (le contenu)
	local autoSet = self.editForm:FindChild("ChekBoxAuto"):IsChecked() -- On regarde si la CheckBox "Set Automatic" est coche 
	
	if autoSet and self.autoConfiguration == nil then -- Si elle est coche mais qu'aucune configuration n'a t entre, alors on en cre une par dfaut
		self.autoConfiguration = {EnableRepeat = false, EnableSchedule = false, EnableZoneChanged = false, Zone = "", RepeatCounter = 0, LimitCounter = 0, Limit = 0, RepeatTime = "", ScheduleOn = "12:00"}
	end
	
	if titleText ~= "" and contentText ~= "" then -- On ne ralise l'ajout que si le titre et le contenu sont prsents
		if self.announcementList[titleText] == nil then
			local rangeOptions = self:ScanRangeOptions() -- On scan les options de porte (guild, say, yell etc...)
			
			self.announcementList[titleText] = {contentText, rangeOptions, autoSet, self.autoConfiguration}	
			
			self:PopulateItemList() -- On rafraichit l'affichage de la liste
			
			self:OnEditClose() -- On ferme la fentre d'ajout
		else
			self:Info("An announcement with this name already exists!")
		end
	else
		self:Info("You may complete the title and the content field!")
	end	
end


-- OnEdit : fonction appele lors de l'appui sur le bouton "Editer" de la fentre principale
--          Elle sert  diter une annonce
function AnnouncementManager:OnEdit( wndHandler, wndControl, eMouseButton )

	if self.selectedItem ~= nil then -- On ne lance l'dition que si un lment est slectionn
		self:OpenEditForm("Edit") -- On ouvre la fentre d'dition / ajout en mode dition
		
		self.editingItem = self.selectedItem:FindChild("Text"):GetText() -- On sauvegarde le titre de l'annonce en cours d'dition
		local originalContent = self.announcementList[self.editingItem][1]
		
		-- On renseigne les champs de la fentre d'dition avec les valeurs correspondants  l'annonce que l'on veut modifier	
		self.editForm:FindChild("EditTextTitle"):FindChild("EditBox"):SetText(self.editingItem)
		self.editForm:FindChild("EditTextContent"):FindChild("EditBox"):SetText(originalContent)
		
		self:SetRangeInfos(self.announcementList[self.editingItem][2], self.announcementList[self.editingItem][3]) -- On place les coches aux options appropries
		self.autoConfiguration = self.announcementList[self.editingItem][4] -- On met en place la configuration courrante
			
		if self.announcementList[self.editingItem][3] then -- Si l'annonce est automatique alors on active le bouton de configuration
			self.editForm:FindChild("BtnAuto"):Enable(true)
		end
		
	else
		self:Info("You have to select an announcement before doing that!")	
	end
end


-- onEditAnnouncement : fonction permettant l'dition d'une annonce
function AnnouncementManager:onEditAnnouncement()
	self.announcementList[self.editingItem] = nil -- On supprime l'annonce en cours d'dition
	self:addAnnouncement()	-- Puis on en recre une nouvelle avec les modifications apportes
end


-- Info : Fonction permettant d'afficher un message dans le chat  l'utilisateur
-- message : string (message  afficher)
function AnnouncementManager:Info(message)
	ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_System, message, "AnnouncementManager")
end

function AnnouncementManager:InitCB()
	self.CBsay = self.editForm:FindChild("CheckBoxSay")
	self.CByell = self.editForm:FindChild("CheckBoxYell")
	self.CBteam = self.editForm:FindChild("CheckBoxTeam")
	self.CBadvice = self.editForm:FindChild("CheckBoxAdvice")
	self.CBpvp = self.editForm:FindChild("CheckBoxPVP")
	self.CBguild = self.editForm:FindChild("CheckBoxGuild")
	self.CBtrade = self.editForm:FindChild("CheckBoxTrade")
	self.CBinstance = self.editForm:FindChild("CheckBoxInstance")
	self.CBzone = self.editForm:FindChild("CheckBoxZone")
	self.CBcircles = self.editForm:FindChild("CheckBoxCircles")
end

-- ScanRangeOptions : Permet de scanner les options de porte
function AnnouncementManager:ScanRangeOptions()

	local rangeOptions = {}
	
	if self.CBsay:IsChecked() then rangeOptions["say"] = true end
	
	if self.CByell:IsChecked() then rangeOptions["yell"] = true end

	if self.CBteam:IsChecked() then rangeOptions["team"] = true end

	if self.CBadvice:IsChecked() then rangeOptions["advice"] = true end

	if self.CBpvp:IsChecked() then rangeOptions["pvp"] = true end
	
	if self.CBguild:IsChecked() then rangeOptions["guild"] = true end

	if self.CBtrade:IsChecked() then rangeOptions["trade"] = true end
	
	if self.CBinstance:IsChecked() then rangeOptions["instance"] = true end
		
	if self.CBzone:IsChecked() then rangeOptions["zone"] = true end	
	
	if self.CBcircles:IsChecked() then rangeOptions["circles"] = true end
	
	rangeOptions["circlesList"] = self.editForm:FindChild("EditBoxCircles"):GetText()
	
	return rangeOptions	
end

-- UnCheckAll : Permet de rinitialiser les coches des options de porte
function AnnouncementManager:UnCheckAll()

	self.CBsay:SetCheck(false)
	self.CByell:SetCheck(false)
	self.CBteam:SetCheck(false)
	self.CBadvice:SetCheck(false)
	self.CBpvp:SetCheck(false)
	self.CBguild:SetCheck(false)
	self.CBtrade:SetCheck(false)
	self.CBinstance:SetCheck(false)
	self.CBzone:SetCheck(false)
	self.CBcircles:SetCheck(false)
	
	self:DisableCircles()
	self.editForm:FindChild("ChekBoxAuto"):SetCheck(false)
	
end

-- SetRangeInfos : Permet de placer les coches des options de porte l o il faut
function AnnouncementManager:SetRangeInfos(rangeOptions, autoChecked)
	
	if rangeOptions["say"] then  self.CBsay:SetCheck(true) end
	if rangeOptions["yell"] then  self.CByell:SetCheck(true) end
	if rangeOptions["team"] then  self.CBteam:SetCheck(true) end
	if rangeOptions["advice"] then  self.CBadvice:SetCheck(true) end
	if rangeOptions["pvp"] then  self.CBpvp:SetCheck(true) end
	if rangeOptions["guild"] then  self.CBguild:SetCheck(true) end
	if rangeOptions["trade"] then  self.CBtrade:SetCheck(true) end
	if rangeOptions["instance"] then  self.CBinstance:SetCheck(true) end
	if rangeOptions["zone"] then  self.CBzone:SetCheck(true) end
	
	if rangeOptions["circles"] then  
		self.CBcircles:SetCheck(true)
		self:EnableCircles() 
		self.editForm:FindChild("EditBoxCircles"):SetText(rangeOptions["circlesList"])	
	end
	
	if autoChecked then self.editForm:FindChild("ChekBoxAuto"):SetCheck(true) end

end


-- Announce : Fonction permettant de poster une annonce sur les channels souhait
-- title : titre de l'annonce  poster
function AnnouncementManager:Announce(title)

	local content = self.announcementList[title][1]
	local rangeOptions = self.announcementList[title][2]
	local newCircleList = ""
	
	if rangeOptions["circles"] then
		newCircleList = string.upper(string.gsub(rangeOptions["circlesList"], " ", ""))
	end
	
	for k, v in pairs(ChatSystemLib.GetChannels()) do
	
		if rangeOptions["say"] and v:GetType() == ChatSystemLib.ChatChannel_Say then v:Send(content) end
		if rangeOptions["yell"] and v:GetType() == ChatSystemLib.ChatChannel_Yell then v:Send(content) end
		if rangeOptions["guild"] and v:GetName() == GameLib.GetPlayerUnit():GetGuildName() then v:Send(content) end -- On triche car ChatSystemLib.ChatChannel_Guild ne fonctionne pas
		if rangeOptions["advice"] and v:GetType() == ChatSystemLib.ChatChannel_Advice then v:Send(content) end
		if rangeOptions["party"] and v:GetType() == ChatSystemLib.ChatChannel_Party then v:Send(content) end
		if rangeOptions["pvp"] and v:GetType() == ChatSystemLib.ChatChannel_ZonePvP then v:Send(content) end
		if rangeOptions["trade"] and v:GetType() == ChatSystemLib.ChatChannel_Trade then v:Send(content) end
		if rangeOptions["instance"] and v:GetType() == ChatSystemLib.ChatChannel_Instance then v:Send(content) end
		if rangeOptions["zone"] and v:GetType() == ChatSystemLib.ChatChannel_Zone then v:Send(content) end
		
		if rangeOptions["circles"] and v:GetType() == ChatSystemLib.ChatChannel_Society then
			if newCircleList == "" or string.find(newCircleList, string.upper(string.gsub(v:GetName(), " ", ""))) then
				v:Send(content)
			end
		end
		
	end
end

-- CheckDigits : Fonction permettant d'afficher des chiffres sur deux digits
-- number : Chiffre  afficher sur deux digits
function AnnouncementManager:CheckDigits(number)

	local newNumber = number
	
	if tonumber(number) < 10 then
		newNumber = "0" .. number
	end
	
	return newNumber 
end

-- OnLink : Apple lorqu'un lien actif est effectu. Permet de copier le lien dans le contenu de l'annonce
-- link : Lien effectu
function AnnouncementManager:OnLink(link)
	
	if link == nil then
		return
	end
	
	local linkText = link:GetChatLinkString() -- On rcupre la chane  mettre dans le chat
	
	if self.editForm:IsShown() then -- Si la fentre d'dition est ouverte on traite le lien
		
		local wndEdit = self.editForm:FindChild("EditTextContent"):FindChild("EditBox")
		local tSelectedText = wndEdit:GetText()

		wndEdit:SetText(tSelectedText .. linkText)
	end
	
end

-- CheckZoneChanged : Appele toutes les 1 secondes pour vrifier si la map a chang. Permet d'effectuer l'annonce de toutes les annonces rgles sur "Post when zone changed :"
function AnnouncementManager:CheckZoneChanged()

	local currentMap = GameLib.GetCurrentZoneMap()
	
	if currentMap ~= nil then
		
		local currentZoneName = string.lower(currentMap.strName)
		
		if self.currentMap ~= currentZoneName then
		 
			self.currentMap = currentZoneName 
			
			for title, content in pairs(self.announcementList) do
				if content[3] then
					if content[4].EnableZoneChanged and (currentZoneName == string.lower(content[4].Zone) or content[4].Zone == "") and (content[4].Limit == 0 or tonumber(content[4].LimitCounter) >= tonumber(content[4].Limit)) then
						self:Announce(title)
						content[4].LimitCounter = 0
					end
				end
			end
		end
	end
end

-- HandleSchedule : Appele toutes les minutes sur les annonces Scheduled. Sert  poster les annonces si l'heure programme est atteinte
-- title : Titre de l'annonce  traiter
-- autoConfig : Configuration de l'annonce automatique  traiter
-- time : Heure locale
function AnnouncementManager:HandleSchedule(title, autoConfig, time)
	if autoConfig.ScheduleOn == time then
		self:Announce(title)
	end
end

-- HandleRepeat : Appele toutes les minutes sur les annonces en Repeat. Sert  poster les annonces  la frquence programme
-- title : Titre de l'annonce  traiter
-- autoConfig : Configuration de l'annonce automatique  traiter
function AnnouncementManager:HandleRepeat(title, autoConfig)

	autoConfig.RepeatCounter = autoConfig.RepeatCounter + 1
	
	if autoConfig.RepeatCounter == tonumber(autoConfig.RepeatTime) then
		autoConfig.RepeatCounter = 0
		self:Announce(title)
	end
end


-----------------------------------------------------------------------------------------------
-- AnnouncementManager Instance
-----------------------------------------------------------------------------------------------
local AnnouncementManagerInst = AnnouncementManager:new()
AnnouncementManagerInst:Init()
